#!/usr/bin/python
# -*- coding:utf-8; tab-width:4; mode:python -*-

import argparse

from configobj import ConfigObj
from validate import Validator
from commodity.pattern import MetaBunch
from commodity.type_ import module_to_dict

import config

defaults = ConfigObj(configspec=file('config.spec'))
defaults.validate(Validator(), copy=True)

defa = MetaBunch(defaults)

parser = argparse.ArgumentParser(prog='prego', prefix_chars='+-')

# behaviour
parser.add_argument('+k', '++keep-going', action='store_true',
                    help="continue even with failed assertion or tests")

parser.add_argument('+d', '++dirty', action='store_true',
                    help="do not remove generated files")

# output
parser.add_argument('+o', '++stdout', action='store_true',
                    help='print tests stdout')
parser.add_argument('+e', '++stderr', action='store_true',
                    help='print tests stderr')
parser.add_argument('+v', '++verbose', dest='verbosity', action='count',
                    help='increase log verbosity')

# styling
parser.add_argument('+p', '++plain', action='store_false', dest='color',
                    help='avoid colors and styling in output')

opts, args = parser.parse_known_args(namespace=defa)

print opts
for key, value in opts.items():
    setattr(config, key, value)

print module_to_dict(config), args



# -*- mode: python; coding: utf-8 -*-

from commodity.str_ import Printable
from commodity.type_ import checked_type
from commodity.net import is_port_open, is_host_reachable

from .assertion import Matcher


#-- subjects
class Host(Printable):
    def __init__(self, name):
        self.name = checked_type(str, name)

    def __unicode__(self):
        return unicode(self.name)

localhost = Host('localhost')


#-- matchers
class ListenPort(Matcher):
    def __init__(self, port, proto='tcp'):
        self.port = checked_type(int, port)
        assert 0 < port < 65536
        self.proto = proto

    def _matches(self, item):
        self.host = checked_type(Host, item)
        return is_port_open(self.port, self.proto, self.host.name)

    def describe_to(self, description):
        description.append_text('port {0}/{1} to be open'.format(
                self.port, self.proto))

    def describe_mismatch(self, item, mismatch_description):
        mismatch_description.append_text('it is closed')


def listen_port(port):
    return ListenPort(port)


class Reachable(Matcher):
    def _matches(self, host):
        self.host = checked_type(Host, host)
        return is_host_reachable(str(host))

    def describe_to(self, description):
        description.append_text("host is reachable")


def reachable():
    return Reachable()



# -*- coding:utf-8; tab-width:4; mode:python -*-
'''
mail related subjects and assertions
'''

from commodity.type_ import checked_type
from commodity.os_ import SubProcess
from commodity.str_ import Printable

from .assertion import Matcher


class SMTP(Printable):
    pass



# -*- coding:utf-8; tab-width:4; mode:python -*-
'''
Debian related assertion
'''

from commodity.type_ import checked_type
from commodity.os_ import SubProcess
from commodity.str_ import Printable

from .assertion import Matcher


class Package(Printable):
    def __init__(self, name):
        self.name = checked_type(str, name)

    def __unicode__(self):
        return unicode(self.name)


class DebPackageInstalled(Matcher):
    def _matches(self, package):
        self.package = package
        sp = SubProcess('dpkg -l %s | grep ^ii' % self.package.name, shell=True)
        return not sp.wait()

    def describe_to(self, description):
        description.append_text('package is installed')


def installed():
    return DebPackageInstalled()
